@@ -2,12 +2,26 @@ use strict;
 package Event::timer;
 use Carp;
 use Time::HiRes qw(time);
-BEGIN { 'Event::Loop'->import(qw(PRIO_NORMAL queueEvent)); }
-
-'Event'->register;
+BEGIN { 'Event::Loop'->import(qw(PRIO_NORMAL queueEvent $Now)); }
 
 my @timer = ();
 
+'Event'->register(prepare => sub {
+		      $Now = time();
+		      @timer? $timer[0]->{'when'} - $Now : 3600;
+		  },
+
+		  check => sub {
+		      $Now = time();
+		      while (@timer and $timer[0]->{'when'} <= $Now) {
+			  my $timer = shift @timer;
+			  $timer->{active} = 0;
+			  next if $timer->{'cancelled'};
+			  queueEvent($timer, $timer->{'when'});
+			  $timer->again if $timer->{'repeat'};
+		      }
+		  });
+
 sub new {
 #    lock %Event::;
 
@@ -45,35 +59,6 @@ sub new {
     $o;
 }
 
-sub prepare { 
-    @timer? $timer[0]->{'when'} - time() : 3600;
-}
-
-sub check {
-    my $now = time();
-    while (@timer and $timer[0]->{'when'} <= $now) {
-	my $timer = shift @timer;
-	$timer->{queued} = 0;
-	next if $timer->{'cancelled'};
-
-	my $cb = $timer->{'callback'};
-	my $sub;
-	if (!$Event::DebugLevel) {
-	    $sub = sub {
-		$cb->($timer,$timer->{'when'});
-	    };
-	} else {
-	    $sub = sub {
-		Event::invoking($timer);
-		$cb->($timer,$timer->{'when'});
-		Event::completed($timer);
-	    };
-	}
-	queueEvent($timer->{'priority'}, $sub);
-	$timer->again if $timer->{'repeat'};
-    }
-}
-
 sub _insert {
     my $obj=shift;
 
@@ -115,8 +100,8 @@ sub again {
     croak "$obj->again: no interval specified"
 	unless exists $obj->{'interval'};
 
-    return if $obj->{queued};
-    $obj->{queued} = 1;
+    return if $obj->{active};
+    $obj->{active} = 1;
 
     $obj->{'when'} = $obj->{'interval'} +
 	($obj->{'hard'} ? $obj->{'when'} : time());
